//
//  FDDAController.m
//  DiskArbitrationTest
//
//  Created by Francis Devereux on 27/10/2008.
//  Copyright 2008 Francis Devereux. All rights reserved.
//

#import "FDDAController.h"

@implementation FDDAController

static DADissenterRef DiskEjectApprovalCallback(DADiskRef disk, void *context)
{
    //FDDAController *self = (FDDAController *)context;

    CFDictionaryRef diskDescription = DADiskCopyDescription(disk);
    NSLog(@"DiskEjectApprovalCallback diskDescription = %@", diskDescription);
    CFRelease(diskDescription);
    
    return NULL;
}

static DADissenterRef DiskUnmountApprovalCallback(DADiskRef disk, void *context)
{
    //FDDAController *self = (FDDAController *)context;
    
    OSStatus status;
    OSErr err;
    
    CFDictionaryRef diskDescription = DADiskCopyDescription(disk);
    NSLog(@"DiskUnmountApprovalCallback diskDescription = %@", diskDescription);
    
    NSDictionary *nsDiskDescription = (NSDictionary *)diskDescription;
    NSURL *volumeURL = [nsDiskDescription valueForKey:@"DAVolumePath"];
    if (![volumeURL isFileURL])
    {
        NSLog(@"Error: volume URL %@ is not a file URL!", volumeURL);
        goto EXIT1;
    }
    
    NSString *volumePath = [volumeURL path];
    const char *utf8VolumePath = [volumePath UTF8String];
    FSRef volumeFSRef;
    if ((status = FSPathMakeRef((UInt8 *)utf8VolumePath, &volumeFSRef, (Boolean *)NULL)) != noErr)
    {
        NSLog(@"FDPathMakeRef error: %d", status);
        goto EXIT1;
    }
    
    FSCatalogInfo catalogInfo;
    if ((err = FSGetCatalogInfo(&volumeFSRef, kFSCatInfoVolume, &catalogInfo, NULL, NULL, NULL)) != noErr)
    {
        NSLog(@"FSGetCatalogInfo error: %d", err);
        goto EXIT1;
    }
    
    FSVolumeInfo volumeInfo;
    if ((err = FSGetVolumeInfo(catalogInfo.volume,
                               0, NULL,
                               kFSVolInfoGettableInfo, &volumeInfo,
                               NULL,
                               NULL)) != noErr)
    {
        NSLog(@"FSGetVolumeInfo error: %d", err);
        goto EXIT1;
    }
    
    NSLog(@"totalBytes = %llu, freeBytes = %llu, blockSize = %u", volumeInfo.totalBytes, volumeInfo.freeBytes, volumeInfo.blockSize);
    
EXIT1:
    CFRelease(diskDescription);
    
    return NULL;
}

- (void)awakeFromNib
{
    NSLog(@"FDDAController awakeFromNib");
    
    daApprovalSession = DAApprovalSessionCreate(kCFAllocatorDefault);
    DAApprovalSessionScheduleWithRunLoop(daApprovalSession, 
                                         [[NSRunLoop currentRunLoop] getCFRunLoop],
                                         kCFRunLoopCommonModes);
    
    DARegisterDiskEjectApprovalCallback(daApprovalSession,
                                        NULL, 
                                        &DiskEjectApprovalCallback, 
                                        (void *)self);  
    DARegisterDiskUnmountApprovalCallback(daApprovalSession,
                                          NULL, 
                                          &DiskUnmountApprovalCallback, 
                                          (void *)self);
    NSLog(@"finished registering callbacks");
}

- (void)dealloc
{
    DAUnregisterApprovalCallback(daApprovalSession, DiskEjectApprovalCallback, self);
    DAUnregisterApprovalCallback(daApprovalSession, DiskUnmountApprovalCallback, self);

    DAApprovalSessionUnscheduleFromRunLoop(daApprovalSession, 
                                           [[NSRunLoop currentRunLoop] getCFRunLoop],
                                           kCFRunLoopCommonModes);
    
    CFRelease(daApprovalSession);
    [super dealloc];
}

@end
